@@ -51,9 +51,14 @@ static inline int pclose(FILE *stream)
 #include <qdir.h>
 #include <QtCore/QProcess>
 #include <QtCore/QSet>
+#include <QtCore/QBasicMutex>
+#include <QtCore/QMutexLocker>
 
 bool KConfigPrivate::mappingsRegistered = false;
 
+Q_GLOBAL_STATIC(QStringList, s_globalFiles) // For caching purposes.
+static QBasicMutex s_globalFilesMutex;
+
 KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags,
                                QStandardPaths::StandardLocation resourceType)
     : openFlags(flags), resourceType(resourceType), mBackend(Q_NULLPTR),
@@ -73,7 +78,7 @@ KConfigPrivate::KConfigPrivate(KConfig::OpenFlags flags,
 #ifdef Q_OS_WIN
             QFile::decodeName(qgetenv("WINDIR") + "/kde5rc");
 #else
-            QLatin1String("/etc/kde5rc");
+            QStringLiteral("/etc/kde5rc");
 #endif
         if (!QFileInfo(etc_kderc).isReadable()) {
             etc_kderc.clear();
@@ -216,6 +221,14 @@ QString KConfigPrivate::expandString(const QString &value)
                     QByteArray pEnv = qgetenv(aVarName.toLatin1().constData());
                     if (!pEnv.isEmpty()) {
                         env = QString::fromLocal8Bit(pEnv.constData());
+                    } else {
+                        if (aVarName == QStringLiteral("QT_DATA_HOME")) {
+                            env = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
+                        } else if (aVarName == QStringLiteral("QT_CONFIG_HOME")) {
+                            env = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation);
+                        } else if (aVarName == QStringLiteral("QT_CACHE_HOME")) {
+			    env = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation);
+                        }
                     }
                 }
                 aValue.replace(nDollarPos, nEndPos - nDollarPos, env);
@@ -440,6 +453,12 @@ bool KConfig::sync()
             QExplicitlySharedDataPointer<KConfigBackend> tmp = KConfigBackend::create(d->sGlobalFileName);
             if (d->configState == ReadWrite && !tmp->lock()) {
                 qWarning() << "couldn't lock global file";
+
+                //unlock the local config if we're returning early
+                if (d->mBackend->isLocked()) {
+                    d->mBackend->unlock();
+                }
+
                 d->bDirty = true;
                 return false;
             }
@@ -487,7 +506,7 @@ void KConfig::checkUpdate(const QString &id, const QString &updateFile)
     const QString cfg_id = updateFile + QLatin1Char(':') + id;
     const QStringList ids = cg.readEntry("update_info", QStringList());
     if (!ids.contains(cfg_id)) {
-        QProcess::execute(QStringLiteral(KCONF_UPDATE_INSTALL_LOCATION), QStringList() << QString::fromLatin1("--check") << updateFile);
+        QProcess::execute(QStringLiteral(KCONF_UPDATE_INSTALL_LOCATION), QStringList() << QStringLiteral("--check") << updateFile);
         reparseConfiguration();
     }
 }
@@ -524,23 +543,35 @@ KConfig::OpenFlags KConfig::openFlags() const
     return d->openFlags;
 }
 
-Q_GLOBAL_STATIC(QString, globalMainConfigName)
+struct KConfigStaticData
+{
+    QString globalMainConfigName;
+    // Keep a copy so we can use it in global dtors, after qApp is gone
+    // This is a workaround for the Qt bug fixed in https://codereview.qt-project.org/107005 for Qt 5.5
+    // Revert this addition once we can depend on Qt 5.5.
+    QStringList appArgs;
+};
+Q_GLOBAL_STATIC(KConfigStaticData, globalData)
 
 void KConfig::setMainConfigName(const QString &str)
 {
-    *globalMainConfigName() = str;
+    globalData()->globalMainConfigName = str;
 }
 
 QString KConfig::mainConfigName()
 {
+    KConfigStaticData* data = globalData();
+    if (data->appArgs.isEmpty())
+        data->appArgs = QCoreApplication::arguments();
+
     // --config on the command line overrides everything else
-    const QStringList args = QCoreApplication::arguments();
+    const QStringList args = data->appArgs;
     for (int i = 1; i < args.count(); ++i) {
         if (args.at(i) == QLatin1String("--config") && i < args.count() - 1) {
             return args.at(i + 1);
         }
     }
-    const QString globalName = *globalMainConfigName();
+    const QString globalName = data->globalMainConfigName;
     if (!globalName.isEmpty()) {
         return globalName;
     }
@@ -560,7 +591,7 @@ void KConfigPrivate::changeFileName(const QString &name)
             file = QStandardPaths::writableLocation(resourceType) + QLatin1Char('/') + fileName;
         } else if (wantGlobals()) { // accessing "kdeglobals" by specifying no filename and NoCascade - XXX used anywhere?
             resourceType = QStandardPaths::GenericConfigLocation;
-            fileName = QLatin1String("kdeglobals");
+            fileName = QStringLiteral("kdeglobals");
             file = sGlobalFileName;
         } else {
             // anonymous config
@@ -610,6 +641,11 @@ void KConfig::reparseConfiguration()
 
     d->bFileImmutable = false;
 
+    {
+        QMutexLocker locker(&s_globalFilesMutex);
+        s_globalFiles()->clear();
+    }
+
     // Parse all desired files from the least to the most specific.
     if (d->wantGlobals()) {
         d->parseGlobalFiles();
@@ -620,17 +656,27 @@ void KConfig::reparseConfiguration()
 
 QStringList KConfigPrivate::getGlobalFiles() const
 {
-    QStringList globalFiles;
-    Q_FOREACH (const QString &dir1, QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QLatin1String("kdeglobals"))) {
-        globalFiles.push_front(dir1);
+    QMutexLocker locker(&s_globalFilesMutex);
+    if (s_globalFiles()->isEmpty()) {
+        const QStringList paths1 = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QStringLiteral("kdeglobals"));
+        const QStringList paths2 = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QStringLiteral("system.kdeglobals"));
+
+        const bool useEtcKderc = !etc_kderc.isEmpty();
+        s_globalFiles()->reserve(paths1.size() + paths2.size() + (useEtcKderc ? 1 : 0));
+
+        Q_FOREACH (const QString &dir1, paths1) {
+            s_globalFiles()->push_front(dir1);
+        }
+        Q_FOREACH (const QString &dir2, paths2) {
+            s_globalFiles()->push_front(dir2);
+        }
+
+        if (useEtcKderc) {
+            s_globalFiles()->push_front(etc_kderc);
+        }
     }
-    Q_FOREACH (const QString &dir2, QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QLatin1String("system.kdeglobals"))) {
-        globalFiles.push_front(dir2);
-    }
-    if (!etc_kderc.isEmpty()) {
-        globalFiles.push_front(etc_kderc);
-    }
-    return globalFiles;
+
+    return *s_globalFiles();
 }
 
 void KConfigPrivate::parseGlobalFiles()
@@ -675,6 +721,12 @@ void KConfigPrivate::parseConfigFiles()
                     Q_FOREACH (const QString &f, QStandardPaths::locateAll(resourceType, fileName)) {
                         files.prepend(f);
                     }
+
+                    // allow fallback to config files bundled in resources
+                    const QString resourceFile(QStringLiteral(":/kconfig/") + fileName);
+                    if (QFile::exists(resourceFile)) {
+                        files.prepend(resourceFile);
+                    }
                 }
             }
         } else {
@@ -863,11 +915,11 @@ bool KConfig::isConfigWritable(bool warnUser)
 
         // Note: We don't ask the user if we should not ask this question again because we can't save the answer.
         errorMsg += QCoreApplication::translate("KConfig", "Please contact your system administrator.");
-        QString cmdToExec = QStandardPaths::findExecutable(QString::fromLatin1("kdialog"));
+        QString cmdToExec = QStandardPaths::findExecutable(QStringLiteral("kdialog"));
         if (!cmdToExec.isEmpty()) {
             QProcess::execute(cmdToExec, QStringList()
-                              << QString::fromLatin1("--title") << QCoreApplication::applicationName()
-                              << QString::fromLatin1("--msgbox") << errorMsg);
+                              << QStringLiteral("--title") << QCoreApplication::applicationName()
+                              << QStringLiteral("--msgbox") << errorMsg);
         }
     }
 
